'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var minimist = require('minimist');
var Spinner = require('cli-spinner').Spinner;
var logFormatters = require('./formatters');
var parseCommandOptions = require('./parse');
var deepSet = require('deep-set');

var GLOBAL_OPTIONS = {
  override: ['string'],
  config: 'string'
};

var CliHarness = function () {
  function CliHarness(app) {
    _classCallCheck(this, CliHarness);

    this.app = app;

    this.spinner = new Spinner('%s');
    this.spinner.setSpinnerString('|/-\\');

    hookInto(this, app);
  }

  _createClass(CliHarness, [{
    key: 'run',
    value: function run(argv) {
      var parsedArgs = minimist(argv, { boolean: true });

      var commandName = parsedArgs._.shift();
      var command = this.app.command(commandName);

      // TODO: parsing can be moved to its own module and tested more thoroughly
      var allowedOptions = Object.assign({}, command.allowedOptions, GLOBAL_OPTIONS);
      var commandOptions = parseCommandOptions(commandName, allowedOptions, parsedArgs);

      if (commandOptions.config) {
        this.app.setCustomConfigPath(commandOptions.config);
      }

      if (commandOptions.override) {
        this.processOverrides(this.app, commandOptions.override);
      }

      return this.app.run(commandName, commandOptions);
    }
  }, {
    key: 'processOverrides',
    value: function processOverrides(app, overrideAssignments) {
      var config = app.config();
      overrideAssignments.forEach(function (assignment) {
        var _assignment$split = assignment.split('=');

        var _assignment$split2 = _slicedToArray(_assignment$split, 2);

        var path = _assignment$split2[0];
        var value = _assignment$split2[1];

        deepSet(config, path, value);
      });
    }
  }, {
    key: 'onLog',
    value: function onLog(level) {
      var normalizedLevel = level.toLowerCase();

      var formatter = logFormatters[normalizedLevel] || logFormatters.default;

      // eslint-disable-next-line no-console
      var logger = console[normalizedLevel] || console.log;

      // eslint-disable-next-line no-console

      for (var _len = arguments.length, messages = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        messages[_key - 1] = arguments[_key];
      }

      logger.call.apply(logger, [console, formatter('[' + level.toUpperCase() + ']')].concat(messages));
    }
  }, {
    key: 'onWaiting',
    value: function onWaiting(isWaiting) {
      if (isWaiting) {
        this.spinner.start();
      } else {
        this.spinner.stop();
      }
    }
  }]);

  return CliHarness;
}();

function hookInto(harness, app) {
  app.on('log', harness.onLog.bind(harness));
  app.on('waiting', harness.onWaiting.bind(harness));
}

module.exports = CliHarness;