'use strict';

module.exports = parseCommandOptions;

function parseCommandOptions(commandName) {
  var allowedOptions = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var parsedArgv = arguments[2];

  var options = {};

  Object.keys(allowedOptions).forEach(function (key) {
    if (key === '_') {
      return;
    }

    var value = parseOption(commandName, parsedArgv[key], allowedOptions[key]);
    if (value !== undefined) {
      options[key] = value;
    }
  });

  var argumentOptions = allowedOptions._ || [];
  argumentOptions.forEach(function (entry) {
    validateAllowedArgumentEntry(commandName, entry);

    var value = parseOption(commandName, parsedArgv._.shift(), entry.type);
    if (value !== undefined) {
      options[entry.name] = value;
    }
  });

  return options;
}

function parseOption(commandName, value, type) {
  if (!type) {
    return value;
  }

  if (type instanceof Function) {
    return type(value);
  }

  if (type instanceof Array) {
    if (!type[0]) {
      throw new Error('Invalid command line option type in command \'' + commandName + '\': array option ' + 'has no inner option type (ie, should be set to something like [\'string\']).');
    }

    if (!value) {
      return [];
    }

    var arrayValue = value instanceof Array ? value : [value];
    return arrayValue.map(function (v) {
      return parseOption(commandName, v, type[0]);
    });
  }

  switch (type) {
    case 'int':
      return parseInt(value, 10);
    case 'string':
      return value ? value.toString() : value;
    case 'float':
      return parseFloat(value);
    case 'bool':
      return !!value;
    default:
      throw new Error('Unknown command line option type \'' + type + '\' in command \'' + commandName + '\'.');
  }
}

function validateAllowedArgumentEntry(commandName, entry) {
  if (!entry.name) {
    throw new Error('Command ' + commandName + ' has invalid allowedOption._ entry: \'name\' property is missing');
  }
}