const path = require('path')
const expect = require('chai').expect
const mockRequire = require('mock-require')
const intercept = require('intercept-stdout')
const Bombastic = require('../../../lib/app')
const CliHarness = require('../../../lib/cli/harness')

const TEST_PROJECT_PATH = path.join(__dirname, '../../resources')

describe('command: deploy', function () {
  let app
  beforeEach(function () {
    app = new Bombastic(TEST_PROJECT_PATH)
  })

  let harness
  beforeEach(function () {
    harness = new CliHarness(app)
  })

  let capturedText
  let unhookIntercept
  function captureText() {
    capturedText = ''
    unhookIntercept = intercept(function (str) {
      capturedText += str
      return ''
    })
  }

  let commandOptions
  beforeEach(function () {
    commandOptions = null

    mockModule('bombastic-deploy-my-deploy-target')
  })
  afterEach(function () {
    mockRequire.stopAll()
  })

  it('should fail if no platform is specified', async function () {
    let error
    try {
      await harness.run(['deploy'])
    } catch (e) {
      error = e
    }

    expect(error).to.be.instanceof(Error)
    expect(error.message).to.match(/no platform specified/i)
  })

  it('should perform a teardown if the teardown option is supplied', async function () {
    captureText()
    try {
      await harness.run(['deploy', 'my-deploy-target', '--teardown'])
    } finally {
      unhookIntercept()
    }

    expect(commandOptions).to.deep.equal({ teardown: true })
    expect(capturedText).to.equal('')
  })

  function mockModule(moduleName) {
    const mockModuleFunction = function (project, options) {
      commandOptions = options
      return Promise.resolve()
    }

    mockRequire(moduleName, mockModuleFunction)
    return mockModuleFunction
  }
})
