const expect = require('chai').expect
const sinon = require('sinon')
const mockRequire = require('mock-require')
const intercept = require('intercept-stdout')
const Bombastic = require('../../../lib/app')
const CliHarness = require('../../../lib/cli/harness')

describe('command: serve', function () {
  let app
  beforeEach(function () {
    app = new Bombastic()
  })

  let harness
  beforeEach(function () {
    harness = new CliHarness(app)
  })

  let capturedText
  let unhookIntercept
  function captureText() {
    capturedText = ''
    unhookIntercept = intercept(function (str) {
      capturedText += str
      return ''
    })
  }

  afterEach(function () {
    mockRequire.stopAll()
  })

  it('should fail when no third argument is supplied', async function () {
    let error
    try {
      await harness.run(['serve'])
    } catch (e) {
      error = e
    }

    expect(error).to.be.instanceof(Error)
    expect(error.message).to.match(/no framework specified/i)
  })

  it('should require and invoke the correct module when run', async function () {
    const EXPECTED_OUTPUT = ''

    const module = mockModule('bombastic-serve-my-test-framework')

    captureText()
    try {
      await app.run('serve', { framework: 'my-test-framework' })
    } finally {
      unhookIntercept()
    }

    expect(module.called).to.be.true
    expect(capturedText).to.equal(EXPECTED_OUTPUT)
  })

  function mockModule(moduleName) {
    const mockModuleFunction = sinon.stub().resolves()
    mockRequire(moduleName, mockModuleFunction)
    return mockModuleFunction
  }
})
