const path = require('path')
const expect = require('chai').expect
const loadSwaggerSpec = require('../../../lib/swagger/load')
const RouteHandler = require('../../../lib/swagger/route-handler')

const TEST_YAML_FILE = path.join(__dirname, '../../resources/swagger.yaml')
const NO_ROUTE_HANDLER_YAML_FILE = path.join(__dirname, '../../resources/no-route-handler.yaml')
const INVALID_ROUTE_HANDLER_YAML_FILE = path.join(__dirname, '../../resources/invalid-route-handler.yaml')

describe('SwaggerSpecification', function () {
  describe('#load()', function () {
    it('should parse the YAML file correctly', async function () {
      const spec = await loadSwaggerSpec(TEST_YAML_FILE)

      expect(spec).to.be.ok

      checkRouteHandler(spec.paths['/pets'].get, 'getAll', 'api/pets')
      checkRouteHandler(spec.paths['/pets'].post, 'create', 'api/pets')
      checkRouteHandler(spec.paths['/pets/{petId}'].get, 'get', 'api/pets')
    })

    it('should fail when x-route-handler fields are missing', async function () {
      let error
      try {
        await loadSwaggerSpec(NO_ROUTE_HANDLER_YAML_FILE)
      } catch (e) {
        error = e
      }

      expect(error).to.be.instanceof(Error)
      expect(error.message).to.match(/no x-route-handler found/i)
    })

    it('should fail when x-route-handler fields are malformed', async function () {
      let error
      try {
        await loadSwaggerSpec(INVALID_ROUTE_HANDLER_YAML_FILE)
      } catch (e) {
        error = e
      }

      expect(error).to.be.instanceof(Error)
      expect(error.message).to.match(/invalid route handler/i)
    })

    function checkRouteHandler(route, expectedMethod, expectedPath) {
      expect(route['x-route-handler']).to.be.instanceof(RouteHandler)
      expect(route['x-route-handler'].method).to.equal(expectedMethod)
      expect(route['x-route-handler'].path).to.equal(expectedPath)
    }
  })
})
